import sqlalchemy as sa
from rasax.community.database import utils

from rasax.community.database.base import Base, Timestamps
from sqlalchemy.orm import relationship


class BusinessLogicFlow(Base, Timestamps):
    """Represents a Business Logic Flow of the assistant."""

    __tablename__ = "business_logic_flow"

    id = sa.Column(sa.Integer, utils.create_sequence(__tablename__), primary_key=True)
    name = sa.Column(sa.String, nullable=False, unique=True)
    elements = relationship("BusinessLogicFlowElement", cascade="all")


class BusinessLogicFlowElement(Base, Timestamps):
    """Represents a Business Logic Flow Element of the assistant."""

    __tablename__ = "business_logic_flow_element"

    id = sa.Column(sa.Integer, utils.create_sequence(__tablename__), primary_key=True)
    type = sa.Column(sa.String, nullable=False)
    element_id = sa.Column(sa.String, nullable=False)
    data = sa.Column(sa.Text, nullable=False)
    business_logic_flow_id = sa.Column(
        sa.Integer, sa.ForeignKey("business_logic_flow.id")
    )
    flow = relationship("BusinessLogicFlow")


class ExampleConversation(Base, Timestamps):
    """Represents an Example Conversation of the assistant."""

    __tablename__ = "example_conversation"

    id = sa.Column(sa.Integer, utils.create_sequence(__tablename__), primary_key=True)
    name = sa.Column(sa.String, nullable=False, unique=True)
    items = relationship("ExampleConversationItem", cascade="all")


class ExampleConversationItem(Base, Timestamps):
    """Represents an Example Conversation Item of the assistant."""

    __tablename__ = "example_conversation_item"

    id = sa.Column(sa.Integer, utils.create_sequence(__tablename__), primary_key=True)
    type = sa.Column(sa.String, nullable=False)
    text = sa.Column(sa.Text, nullable=False)
    example_conversation_id = sa.Column(
        sa.Integer, sa.ForeignKey("example_conversation.id")
    )
    conversation = relationship("ExampleConversation")
